﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Microsoft.AspNet.SignalR;
using System.Threading.Tasks;
using System.Collections;
using System.Diagnostics;
using System.Threading;

namespace SignalR
{
    public class Connection : Hub
    {
        public static List<Player> Players = new List<Player>();
        public static int W = 960;
        public static int H = 640;
        public static List<string> Map = new List<string>();
        public static List<string> Color = new List<string> { "red", "cyan", "lime", "yellow" };
        public static int[,] Depart;
        public static bool Continuer = true;

        public Connection()
        {
            Depart = new int[4, 4] { { 0, H / 2, 1, 0 }, { W / 2, 0, 0, 1 }, { W, H / 2, -1, 0 }, { W / 2, H, 0, -1 } };
        }

        internal static void ClientPlayers(ArrayList a)
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<Connection>();
            context.Clients.All.players(a);
        }

        internal static void ClientWinner(string p)
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<Connection>();
            context.Clients.All.winner(p);
        }

        internal static void ClientReset()
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<Connection>();
            context.Clients.All.reset();
        }

        internal static void ClientplayersOnServer(List<Player> p)
        {
            IHubContext context = GlobalHost.ConnectionManager.GetHubContext<Connection>();
            context.Clients.All.playersOnServer(p);
        }



        public static void Main()
        {
            while (true)
            {
                var setPlayers = new ArrayList();
                foreach (var i in Players)
                {
                    if (i.active)
                    {
                        i.parcours.Push(new[] { i.x, i.y });

                        i.xa = i.x;
                        i.ya = i.y;

                        i.x += i.dx * i.vx;
                        i.y += i.dy * i.vy;
                        
                        if (Map.Contains(i.x + "-" + i.y) || i.x < 0 || i.x > W || i.y < 0 || i.y > H)
                        {
                            i.active = false;
                        }
                        else
                        {
                            Map.Add(i.x + "-" + i.y);
                        }

                    }
                    setPlayers.Add(new PlayerForClient(i.ConnectionId, i.x, i.y, i.xa, i.ya, i.color, i.active));
                    
                }
                
                Thread.Sleep(64);
                ClientPlayers(setPlayers);
                if(verificationStatutDeLaPartie())
                {
                    Thread.Sleep(2000);
                }
            }
        }

        public static bool verificationStatutDeLaPartie()
        {
            var j = Players.Count(p => p.active);
            if (j == 1 && Players.Count() >= 2 || j == 0)
            {
                var winner = Players.Any(p => p.active) ? Players.Single(p => p.active).color : "Nobody";
                ClientWinner(winner);
                Thread.Sleep(2000);
                for (int i = 0; i < Players.Count; i++)
                {
                    Players[i].active = true;
                    Players[i].x = Depart[i % 4, 0];
                    Players[i].y = Depart[i % 4, 1];
                    Players[i].xa = 0;
                    Players[i].ya = 0;
                    Players[i].dx = Depart[i % 4, 2];
                    Players[i].dy = Depart[i % 4, 3];
                    Players[i].parcours.Clear();
                }
                
                Map.Clear();
                ClientReset();
                return true;
            }
            return false;
        }

        public override Task OnConnected()
        {
            var player = new Player(Context.ConnectionId,
                                    Depart[Players.Count % 4, 0],
                                    Depart[Players.Count % 4, 1],
                                    Color[Players.Count % 4],
                                    Depart[Players.Count % 4, 2],
                                    Depart[Players.Count % 4, 3]);

            Players.Add(player);
            Clients.Caller.newPlayer(Context.ConnectionId);
            Clients.Caller.playersOnServer(Players);

            return base.OnConnected();
        }

        public override Task OnDisconnected()
        {
            Players.Remove(Players.Find(p => p.ConnectionId == Context.ConnectionId));
            return base.OnDisconnected();
        }

        public void Down(string Id)
        {
            var player = Players.Find(p => p.ConnectionId == Id);
            if (player.dx != 0)
            {
                player.dx = 0;
                player.dy = 1;
            }
        }

        public void Up(string id)
        {
            var player = Players.Find(p => p.ConnectionId == id);
            if (player.dx != 0)
            {
                player.dx = 0;
                player.dy = -1;
            }
        }

        public void Right(string Id)
        {
            var player = Players.Find(p => p.ConnectionId == Id);
            if (player.dy != 0)
            {
                player.dx = 1;
                player.dy = 0;
            }
        }

        public void Left(string Id)
        {
            var player = Players.Find(p => p.ConnectionId == Id);
            if (player.dy != 0)
            {
                player.dx = -1;
                player.dy = 0;
            }
        }
    }


    public class PlayerForClient
    {
        public string ConnectionId { get; set; }

        public int x { get; set; }
        public int y { get; set; }

        public int xa { get; set; }
        public int ya { get; set; }

        public string color { get; set; }
        public bool active { get; set; }

        public PlayerForClient(string ConnectionId, int x, int y, int xa, int ya, string color, bool active)
        {
            this.ConnectionId = ConnectionId;
            this.x = x;
            this.y = y;
            this.xa = xa;
            this.ya = ya;
            this.color = color;
            this.active = true;
        }
    }

    public class Player
    {
        public string ConnectionId { get; set; }

        public int x { get; set; }
        public int y { get; set; }

        public int xa { get; set; }
        public int ya { get; set; }

        public int vx { get; set; }
        public int vy { get; set; }

        public int dx { get; set; }
        public int dy { get; set; }

        public string color { get; set; }
        public Stack parcours { get; set; }
        public bool active { get; set; }

        public Player(string ConnectionId, int x, int y, string color, int dx, int dy)
        {
            this.ConnectionId = ConnectionId;
            this.x = x;
            this.y = y;
            this.xa = x;
            this.ya = y;
            this.vx = ParametrePlayer.playerVx;
            this.vy = ParametrePlayer.playerVy;
            this.dx = dx;
            this.dy = dy;
            this.color = color;
            this.parcours = new Stack();
            this.active = true;
        }
    }

    public static class ParametrePlayer
    {
        public static int playerX = 6;
        public static int playerY = 100;
        public static int playerVx = 8;
        public static int playerVy = 8;
        public static int playerDx = 1;
        public static int playerDy = 0;
        public static int largeur = 2;

    }

}